<?php

/**
 * Generate a debian file.
 */
function dh_debian_file($file) {
  $contents = file_exists($file) ? file_get_contents($file) : '';
  $filename = preg_replace('/[^\w0-9_]/', '_', basename($file));
  if (strpos('.', $filename) !== FALSE) {
    $filename = substr($filename, 0, strpos('.', $filename));
  }
  if (in_array($filename, array('templates', 'cron', 'libcommon', 'dirs'))) {
    foreach (module_implements('debian_' . $filename) as $module) {
      $contents .= PHP_EOL . module_invoke($module, 'debian_' . $filename);
    }
    return $contents;
  }
  $function = 'dh_debian_file_' . $filename;
  return function_exists($function) ? call_user_func($function, $contents) : FALSE;
}

/**
 * Generate debian file 'config'.
 */
function dh_debian_file_config($contents) {
  $code = PHP_EOL;
  foreach (module_implements('debian_config') as $module) {
    $code .= "# Implemented by $module." . PHP_EOL;
    $code .= module_invoke($module, 'debian_config') . PHP_EOL;
    $code .= "# Implementation by $module finished." . PHP_EOL;
  }
  return strtr($contents, array(
    '#__MODULE_HOOK__' => $code,
    '#__SSL_SUPPORT__' => 'SSL_SUPPORT=' . (drush_get_option('dh-build-ssl', FALSE) ? '1' : '0') . PHP_EOL,
  ));
}

/**
 * Generate debian file 'postinst'.
 */
function dh_debian_file_postinst($contents) {
  $code = PHP_EOL;
  foreach (module_implements('debian_postinst') as $module) {
    $code .= "    // Implemented by $module." . PHP_EOL;
    $code .= module_invoke($module, 'debian_postinst') . PHP_EOL;
    $code .= "    // Implementation by $module finished." . PHP_EOL;
  }
  return str_replace('//__MODULE_HOOK__', $code, $contents);
}

/**
 * Generate debian file 'control'.
 */
function dh_debian_file_control() {
  $control = array(
    'Source' => drush_get_option('dh-package'),
    'Section' => 'web',
    'Priority' => 'optional',
    'Maintainer' => dt('!user <!email>', array('!user' => drush_get_option('dh-email'), '!email' => drush_get_option('dh-user'))),
    'Build-Depends' => array('debhelper (>= 5)', 'dctrl-tools'),
    'Standards-Version' => '3.7.2',
    DH_CONTROL_BREAK,
    'Package' => drush_get_option('dh-package'),
    'Architecture' => 'all',
    'Depends' => array('${misc:Depends}', 'apache2 | nginx', 'cron', 'php5-pgsql | php5-mysql', 'mysql-client | postgresql-client', 'libapache2-mod-php5 | php5-cgi'),
    'Recommends' => array('drush'),
    'Suggests' => array(),
    'Pre-Depends' => array('php5-cli', 'drush'),
    'Description' => dt('!package is a Drupal !version site maintained by !user', array(
      '!package' => drush_get_option('dh-package'),
      '!user' => drush_get_option('dh-user'),
      '!version' => drush_drupal_major_version())
    ),
  );
  drupal_alter('debian_control', $control);

  foreach ($control as $key => &$values) {
    if (empty($values)) {
      unset($control[$key]);
      continue;
    }
    if ($values == DH_CONTROL_BREAK) {
      $values = '';
      continue;
    }
    if (is_array($values)) {
      $values = implode(', ', $values);
    }
    $values = $key . ': ' . $values;
  }
  return implode(PHP_EOL, $control) . PHP_EOL;
}


/**
 * Generate debian file 'changelog'.
 */
function dh_debian_file_changelog($contents) {
  // If the changelog is new we must add an entry.
  if (empty($contents)) {
    drush_set_option('dh-bump-version', TRUE);
  }
  // If we don't want to change the version, then leave alone.
  if (!drush_get_option('dh-version') && !drush_get_option('dh-bump-version')) {
    // Try to get the existing version.
    preg_match('/^[^\s]+\s\(([^\s]+)\)/', $contents, $matches);
    drush_set_option('dh-version', $matches[1]);
    return $contents;
  }
  // Automate the versioning if specified.
  if (!drush_get_option('dh-version')) {
    $date = date('Ymd');
    $suffix = 1;
    if (preg_match_all('/' . preg_quote($date) . '-(\d+)/', $contents, $matches)) {
      foreach ($matches[1] as $i) {
        $suffix = ($i > $suffix) ? $i : $suffix;
      }
      if (drush_get_option('dh-bump-version')) {
        $suffix++;
      }
    }
    drush_set_option('dh-version', $date . '-' . $suffix);
  }
  drush_print(dt("Debian Changelog. Please enter the changes since last release."));
  drush_print(dt("  Use commas to seperate lines (changelog bullet points)"));
  $message = drush_prompt("Changelog entry");
  if (empty($message)) {
    $message = 'dh-make-drupal packaging script automated build.';
  }
  // Turn commas into bullet points.
  $messages = array_filter(array_filter(explode(',', $message), 'trim'));
  foreach (module_implements('debian_changelog') as $module) {
    $messages += module_invoke($module, 'debian_changelog');
  }
  $messages = '  * ' . implode(PHP_EOL . '  * ', $messages) . PHP_EOL;

  $entry = drush_get_option('dh-package') . ' (' . drush_get_option('dh-version') . ') unstable; urgency=low' . PHP_EOL . PHP_EOL;
  $entry .= $messages . PHP_EOL;
  $entry .= ' -- ' . drush_get_option('dh-user') . ' <' . drush_get_option('dh-email') . '>  ' . date('D, j M Y H:i:s O') . PHP_EOL . PHP_EOL;
  return $entry . $contents;
}

/**
 * Generate debian file 'install'.
 */
function dh_debian_file_install($contents) {
  if (!$dh = opendir(drush_get_option('dh-build-dir'))) {
    drush_set_error("Cannot open drupal root directory.");
    return;
  }
  $www = 'var/www/' . drush_get_option('dh-package');
  $install = array();
  // Go through the root directory and include all files and directories in the install file.
  $excluded_files = array(
    '.', '..', '.htaccess', 'sites', '.debian', 'debian', 'Makefile',
    'update.php', 'install.php', 'cron.php', 'CHANGELOG.txt',
  );
  while (($file = readdir($dh)) !== false) {
    // Exclude these files and folders, they should not be in the production docroot.
    if (in_array($file, $excluded_files)) {
      continue;
    }

    $install[] = array($file, $www);
  }
  closedir($dh);

  // The sites folder is excluded so we need to handle it a bit differently.
  $install[] = array('sites/all', $www . '/sites');
  if (file_exists(drush_get_option('dh-build-dir') . '/sites/default/settings.common.php')) {
    $install[] = array('sites/default/settings.common.php', $www . '/sites/default');
  }

  // SSL support. Install an SSL cert/key pair if present as cert.pem in debian folder.
  if (is_file(DRUPAL_ROOT . '/.debian/cert.pem')) {
    drush_set_option('dh-build-ssl', TRUE);
    $install[] = array('debian/cert.pem', 'etc/' . drush_get_option('dh-package'));
  }

  // Add debian configuration files to install file.
  $install[] = array('debian/phptemplates', 'etc/' . drush_get_option('dh-package'));
  $install[] = array('debian/init-script',  'etc/'. drush_get_option('dh-package'));

  // Add libraries we'll need.
  $install[] = array('debian/libdebconf.php', 'usr/share/' . drush_get_option('dh-package'));
  $install[] = array('debian/libcommon.php', 'usr/share/' . drush_get_option('dh-package'));

  // Allow modules to add install files.
  foreach (module_implements('debian_install') as $module) {
    $install = array_merge($install, module_invoke($module, 'debian_install'));
  }

  $colen = 0;
  // Determine the max length required to make a column based structure to the files.
  foreach ($install as $line) {
    $colen = strlen($line[0]) > $colen ? strlen($line[0]) : $colen;
  }
  foreach ($install as &$line) {
    $line[0] = str_pad($line[0], $colen);
    $line = implode(' ', $line);
  }
  return implode(PHP_EOL, $install) . PHP_EOL;
}
