<?php
/**
 * @file
 *  The drush debian helper
 *
 */

define('DH_CONTROL_BREAK', '--break--');
define('DRUSH_DH_MAKE_API', 1);

/**
 * Implementation of hook_drush_help().
 */
function dh_drush_help($section) {
  switch ($section) {
    case 'drush:dh-make-drupal':
      return dt('--dh-user           Name
--dh-email                        Email
--dh-version                      Version to tag release with
--dh-project                      The name of the project
--dh-package                      The name of the debian package. Defaults to drupal-site-[project]
--dh-bump-version                 Automatically bump the package version.
--dh-build-dir                    The path of the build directory
--debug                           Leave build directory intact
--dh-make-depends=module_name     Require module_name is enabled before building package.
--dh-make-recommends=module_name  Recommend module_name is enabled before building package.
--dh-make-conflicts=module_name   Prevent a package build if module_name is enabled.
--dh-cron-frequency               Set the frequency cron should run every hour. Defaults to 12 (every five minutes).
');
    break;
  }
}

/**
 * Implementation of hook_drush_command().
 */
function dh_drush_command() {
  $items['dh-make-drupal'] = array(
    'description' => 'Debianize Drupal into a buildable debian package.',
    'arguments' => array(),
    'callback' => 'drush_dh_make_drupal',
    'aliases' => array('dh-make', 'dmd'),
    'engines' => array(
      'version_control' => 'Integration with VCS in order to easily commit your changes to projects.',
    ),
  );
  return $items;
}

/**
 * Integration with VCS in order to easily commit your changes to projects.
 */
function dh_drush_engine_version_control() {
  return array(
    'git' => array(
      'signature' => 'git rev-parse --git-dir',
      'options' => array(
        '--version-control=git' => 'Quickly add/remove/commit your project changes to Git.',
        '  --gitsync' => 'Automatically add new files to the Git repository and remove deleted files. Caution.',
        '  --gitcommit' => 'Automatically commit changes to Git repository. You must also using the --gitsync option.',
        '  --gitmessage' => 'Override default commit message which is: Drush automatic commit: <the drush command line used>',
      ),
    ),
  );
}

/**
 * Command callback. Show a list of modules and status.
 *
 */
function drush_dh_make_drupal() {
  // Version 1 didn't specify an API version. Therefore if no
  // api parameter is set, assume the project intends to use the
  // version 1 API.
  if (!drush_get_option('dh-api', FALSE)) {
    drush_set_option('dh-api', 1);
  }

  // If the API versions don't match, don't continue.
  if (drush_get_option('dh-api') != DRUSH_DH_MAKE_API) {
    drush_set_error("Drush dh-make API version mismatch");
    drush_print("This version of drush dh-make implements API version " . DRUSH_DH_MAKE_API, 2);
    drush_print("It doesn't look like this project supports this version of the API.", 2);
    drush_print("To build this package with the correct API version try the following commands then try again:", 2);
    drush_print("");
    drush_print("pushd " . dirname(__FILE__), 4);
    drush_print("git checkout -b API-" . drush_get_option('dh-api') . " --track origin/API-" . drush_get_option('dh-api'), 4);
    drush_print("popd", 4);
    drush_print("");
    drush_print("However, if you know this project does support version " . DRUSH_DH_MAKE_API . " of the dh-make API,", 2);
    drush_print("simply add --dh-api=" . DRUSH_DH_MAKE_API . " to your command and try again or add it to your drushrc file:", 2);
    drush_print("");
    drush_print("\$options['dh-api'] = " . DRUSH_DH_MAKE_API . ";", 4);
    return FALSE;
  }

  include_once(dirname(__FILE__) . '/dh.inc');
  include_once(dirname(__FILE__) . '/dh.debian.inc');
  // Check for an existing debian folder. If it exists, don't build.
  // For legacy reasons we don't attempt to remove the directory.
  if (is_dir(DRUPAL_ROOT . '/debian')) {
    return drush_set_error(DRUPAL_ROOT . '/debian already exists. Please remove or rename to .debian.');
  }

  $options = array('general' => array());

  // Before building the project, check if all the required modules
  // are enabled. These modules are listed as dependancies in the site
  // drushrc.php file.
  $depends = drush_get_option('dh-make-depends', array());
  foreach ($depends as $module) {
    if (!module_exists($module)) {
      drush_set_error($module . " is not enabled but is a dependancy of this build.");
    }
  }
  if (drush_get_error()) {
    return;
  }

  // Same as dependancies but with warnings only and are not fatal.
  $recommends = drush_get_option('dh-make-recommends', array());
  foreach ($recommends as $module) {
    if (!module_exists($module)) {
      drush_log($module . " is not enabled but is recommended for this build.", 'warning');
    }
  }

  // Check for module conflicts that should be disabled before build.
  $conflicts = drush_get_option('dh-make-conflicts', array());
  $continue = TRUE;
  foreach ($conflicts as $module) {
    if (module_exists($module)) {
      drush_log($module . " is enabled but is a conflict of this build.", 'warning');
      $continue = FALSE;
    }
  }
  if (!$continue) {
    return;
  }

  // Try load a version control engine.
  if ($vc_engine = drush_pm_include_version_control(DRUPAL_ROOT)) {
    if ($vc_engine->name() == 'git') {
      if (is_file(DRUPAL_ROOT . '/.gitmodules')) {
        return drush_set_error("Cannot build package: Git submodules are not supported.");
      }
    }
    // Tag the release in version control.
    if (!drush_get_option('dh-version', FALSE) && !drush_get_option('dh-bump-version', FALSE)) {
      if (drush_confirm(dt("Do you want to bump and tag the build in !version_control?", array('!version_control' => $vc_engine->name())))) {
        drush_set_option('dh-bump-version', TRUE);
        drush_log(dt('set dh-bump-version to TRUE in your drushrc file to automate this action.'));
      }
    }
  }

  // Set user
  if (!drush_get_option('dh-user')) {
    if (!$vc_engine || (!$name = $vc_engine->getUsername())) {
      $name = drush_prompt("What is your name?");
    }
    drush_set_option('dh-user', $name);
  }

  // Set email
  if (!drush_get_option('dh-email')) {
    if (!$vc_engine || (!$email = $vc_engine->getEmail())) {
      $email = drush_prompt("What is your email?", drush_get_option('dh-user') . '@drupal.org');
    }
    drush_set_option('dh-email', $email);
  }

  // Set project name.
  if (!drush_get_option('dh-project')) {
    $default = array_pop(explode('/', DRUPAL_ROOT));
    $name = drush_prompt("What is the name of the Drupal project?", $default);
    drush_set_option('dh-project', $name);
  }

  if (!drush_get_option('dh-package')) {
    drush_set_option('dh-package', 'drupal-site-' . drush_get_option('dh-project'));
  }

  // Set project web root.
  if (!drush_get_option('wwwroot')) {
    drush_set_option('wwwroot', '/var/www/' . drush_get_option('dh-package'));
  }
  drush_set_option('wwwrootns', substr(drush_get_option('wwwroot'), 1));

  // Change the version in the debian change log and tag it in VCS.
  if (!is_dir(DRUPAL_ROOT . '/.debian') && !mkdir(DRUPAL_ROOT . '/.debian')) {
    return drush_set_error("Cannot create .debian directory");
  }
  $changelog = file_exists(DRUPAL_ROOT . '/.debian/' . 'changelog') ? file_get_contents(DRUPAL_ROOT . '/.debian/' . 'changelog') : '';
  file_put_contents(DRUPAL_ROOT . '/.debian/' . 'changelog', dh_debian_file_changelog($changelog));

  $tag = NULL;

  if ($vc_engine && drush_get_option('dh-bump-version')) {
    $branch = $vc_engine->getBranch(); 
    $tag = drush_get_option('dh-package') . '_' . $branch . '_' . drush_get_option('dh-version');
    $vc_engine->tag($tag);
  }

  // Give modules the option to add any further files to the .debian directory.
  module_invoke_all('dh_make_prebuild', DRUPAL_ROOT . '/.debian');

  // If we're running version control, it would be a nice idea to build
  // the debian package from version control rather than from the existing
  // directory because this current dir may include information we don't want.
  if ($vc_engine) {
    dh_set_root($vc_engine, $tag);
  }
  if (!dh_create_debian()) {
    return FALSE;
  }

  // Move the Makefile over since passive copy missed it.
  $Makefile = drush_get_option('dh-build-dir') . '/Makefile';
  if (!is_file($Makefile)) {
    copy(dirname(__FILE__) . '/share/Makefile', $Makefile);
  }
  // Make sure debian/rules is executable. Older versions of 'make' don't
  // ensure this.
  chmod(drush_get_option('dh-build-dir') . '/debian/rules', 0744);

  // With the debian folder in place and the placheholders filled up, its now
  // time to attempt to build the debian package.
  $dir = exec("pwd");
  drush_print("Building debian package. This may take a few seconds....");
  chdir(drush_get_option('dh-build-dir'));
  if (drush_shell_exec("make")) {
    drush_log(dt("Debian package completed"));
    // We can assume the package was build in the directory above with a standard naming schema.
    // drupal-site-[project]_[date]-[idx]_add.deb Lets try find it.
    $deb = drush_get_option('dh-build-dir') . '/../' . drush_get_option('dh-package') . '_' . drush_get_option('dh-version') . '_all.deb';
    if ($deb = realpath($deb)) {
      drush_print(dt('Debian package build at !file', array('!file' => $deb)));
    }
  }
  else {
    drush_set_error("Debian package failed to build. Run --debug to see full errors.");
  }
  chdir($dir);

  // Clean up debian directory.
  if (!drush_get_context('DRUSH_DEBUG')) {
    if (drush_get_option('dh-build-dir') != DRUPAL_ROOT) {
      drush_shell_exec("rm -rf %s", drush_get_option('dh-build-dir'));
    }
    else {
      drush_shell_exec("rm -rf %s", DRUPAL_ROOT . '/debian');
      drush_shell_exec("rm -rf %s", DRUPAL_ROOT . '/Makefile');
    }
  }
  else {
    drush_print("Debug mode on. Leaving debian folder intact");
  }

  // This only occurs when the version has been bumped.
  if ($vc_engine && drush_get_option('dh-bump-version')) {
    $vc_engine->commit(array('path' => DRUPAL_ROOT));
    if ($vc_engine->name() == 'git') {
      drush_print("If this repository has a remote then please push the tag:");
      drush_print("git push [remote] --tags");
      drush_print("eg. git push origin --tags");
    }
  }
  dh_recommend_drushrc();
}
